package main

import (
	"encoding/json"
	"fmt"
)

type Addr struct {
	Region string `json:"region"`
	Street string `json:"street"`
	No     int    `json:"no"`
}

type Person struct {
	FirstName  string `json:"first_name"` // FirstName <=> firest_name
	LastName   string `json:"last_name"`
	MiddleName string `json:"middle_name,omitempty"` // `json:"-"` 也可以忽略，omitempty 当是零值时，不序列化
	Age        int    `json:"age,string"`            // 指定类型为 字符串
	//isBoy      bool   `json:"is_boy"` // 不能是小写的，否则提示 Struct field 'isBoy' has 'json' tag but is not exported

	// 命名嵌套
	Addr Addr `json:"addr"`

	// 匿名嵌套，和命名嵌套效果一样的
	//Addr `json:"addr"`
}

func main() {
	//jsonString := `{"first_name": "John", "last_name": "Smith", "age": "12"}`
	jsonString := `{"first_name": "John", "middle_name":"x", "last_name": "Smith", "age": "12", "addr": {"region": "shanghai,huangpu", "street": "Street1", "no": 100}}`
	person := new(Person)
	err := json.Unmarshal([]byte(jsonString), person)
	if err != nil {
		fmt.Println(err.Error())
		return
	} //将json数据转为Person Struct
	fmt.Println(person)

	newJson, _ := json.Marshal(person) //将Person Sturct 转为json格式
	fmt.Printf("%s\n", newJson)
}

// Output:
//&{John Smith x 12 {shanghai,huangpu Street1 100}}
//{"first_name":"John","last_name":"Smith","middle_name":"x","age":"12","addr":{"region":"shanghai,huangpu","street":"Street1","no":100}}
